#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define SE stderr
#define PI 3.141592654
#define TOP 0x1
#define BOTTOM 0x2
#define RIGHT 0x4
#define LEFT 0x8
#define EPS 0.000001

double xmin, ymin, xmax, ymax;
double xa, ya, xb, yb;
double xca, yca, xcb, ycb;
double aa,bb,cc;
double pom;
char T;
int res;

FILE *F;
double xpom, ypom, xv, yv, skala;

/* ================================ 
 * SUPPLEMENTARY CODE
 * ================================ */

double PSScaledx(double xx) { return (50+skala*(xx-xv)); }
double PSScaledy(double yy) { return (50+skala*(yy-yv)); }

void WritePSHeader(void){

  fprintf(F,"%%!PS (c) MisoF\n\n");
  
  fprintf(F,"%% ---------\n%% definicie\n%% ---------\n\n");
  fprintf(F,"/cm {72 2.54 div mul} def\n/pis {newpath moveto show} def\n\n");

  fprintf(F,"%% --------\n%% hlavicka\n%% --------\n\n");
  fprintf(F,"/Times-Roman findfont 24 scalefont setfont\n");
  fprintf(F,"(2D grafika - homework #2) 174 800 pis\n\n");

  fprintf(F,"/Times-Roman findfont 16 scalefont setfont\n1.4 setlinewidth\n\n");
  fprintf(F,"(\\(c\\) MisoF) 520 10 pis\n\n");
  fprintf(F,"/Times-Roman findfont 12 scalefont setfont\n1.4 setlinewidth\n\n");

  /* aby nam to nekreslilo zbytocne mimo */
  xpom=xmax-xmin; ypom=ymax-ymin;
  if (xpom>ypom) skala=225/xpom; else skala=225/ypom;
  xv=xmin-(xmax-xmin)/2; 
  yv=ymin-(ymax-ymin)/2; 

  fprintf(F,"/clipon { newpath 50 50 moveto %.5f 50 lineto",50+skala*xpom*2);
  fprintf(F," %.5f %.5f lineto 50 %.5f lineto\n",50+skala*xpom*2,50+skala*ypom*2,50+skala*ypom*2);
  fprintf(F,"closepath clip } def\n\n");
  fprintf(F,"/clipoff { newpath 0 0 moveto 600 0 lineto 600 800");
  fprintf(F," lineto 0 800 lineto closepath clip } def\n\n");

  fprintf(F,"newpath 45 45 moveto 45 40 lineto 45 45 lineto %.5f 45 lineto",PSScaledx(xmin));
  fprintf(F," %.5f 40 lineto %.5f 45 lineto %.5f 45 lineto %.5f 40 lineto",
          PSScaledx(xmin),PSScaledx(xmin),PSScaledx(xmax),PSScaledx(xmax));
  fprintf(F," %.5f 45 lineto 500 45 lineto stroke\n",PSScaledx(xmax));
  fprintf(F,"newpath 45 45 moveto 40 45 lineto 45 45 lineto 45 %.5f lineto",PSScaledy(ymin));
  fprintf(F," 40 %.5f lineto 45 %.5f lineto 45 %.5f lineto 40 %.5f lineto",
          PSScaledy(ymin),PSScaledy(ymin),PSScaledy(ymax),PSScaledy(ymax));
  fprintf(F," 45 %.5f lineto 45 500 lineto stroke\n",PSScaledy(ymax));
 
  fprintf(F,"(%.2f) 50 30 pis\n",xv);
  fprintf(F,"(%.2f) %.5f 30 pis\n",xmin,PSScaledx(xmin)+5);
  fprintf(F,"(%.2f) %.5f 30 pis\n",xmax,PSScaledx(xmax)+5);
 
  fprintf(F,"(%.2f) (%.2f) stringwidth pop 40 exch sub 50 pis\n",yv,yv);
  fprintf(F,"(%.2f) (%.2f) stringwidth pop 40 exch sub %.5f pis\n",ymin,ymin,PSScaledy(ymin)+5);
  fprintf(F,"(%.2f) (%.2f) stringwidth pop 40 exch sub %.5f pis\n",ymax,ymax,PSScaledy(ymax)+5);
  
  fprintf(F,"clipon\n\n");
  
  fprintf(F,"0.8 0.0 0.0 setrgbcolor\n");
  fprintf(F,"newpath %.5f %.5f moveto %.5f %.5f lineto",
            PSScaledx(xmin),PSScaledy(ymin),PSScaledx(xmin),PSScaledy(ymax));
  fprintf(F," %.5f %.5f lineto %.5f %.5f lineto\n",
            PSScaledx(xmax),PSScaledy(ymax),PSScaledx(xmax),PSScaledy(ymin));
  fprintf(F,"closepath stroke\n");
  fprintf(F,"0.0 0.0 0.0 setrgbcolor\n\n");

  /*  fprintf(F,"%% -------------------\n%% popis transformacie\n");
  fprintf(F,"%% -------------------\n\n");
  fprintf(F,"(Transformation:) 300 755 pis\n"); */
}

/* ================================
 * MAIN FUNCTIONS
 * ================================ */

int RatajKod(double x, double y){
  int kod=0;
  if (y>ymax) kod|=TOP;
  if (y<ymin) kod|=BOTTOM;
  if (x>xmax) kod|=RIGHT;
  if (x<xmin) kod|=LEFT;
  return kod;
}

/* pise sa to s R, SutheRland... */
int CohenSutherland(double xs, double ys, double xf, double yf, 
                    double *xcs, double *ycs, double *xcf, double *ycf) {
  int kods, kodf, kodm, menim, prechod;
  double xnew, ynew;
  
  kods=RatajKod(xs,ys); kodf=RatajKod(xf,yf);
  prechod=2;
  while (1) {
    /* kym nemame oba body trivialne vnutri alebo mimo, iterujeme:*/
    if ((kods|kodf)==0) { *xcs=xs; *ycs=ys; *xcf=xf; *ycf=yf; return prechod; }
    if ((kods&kodf)!=0) return 0;
    prechod=1;
    if (kods) { kodm=kods; menim=1; } else { kodm=kodf; menim=2; }
    if (kodm&TOP) { xnew=xs+(xf-xs)*(ymax-ys)/(yf-ys); ynew=ymax; }
      else if (kodm&BOTTOM) { xnew=xs+(xf-xs)*(ymin-ys)/(yf-ys); ynew=ymin; }
      else if (kodm&RIGHT) { xnew=xmax; ynew=ys+(yf-ys)*(xmax-xs)/(xf-xs); }
      else if (kodm&LEFT) { xnew=xmin; ynew=ys+(yf-ys)*(xmin-xs)/(xf-xs); }
    if (menim==1) { xs=xnew; ys=ynew; kods=RatajKod(xs,ys); }
      else { xf=xnew; yf=ynew; kodf=RatajKod(xf,yf); }
  }
}
    
int main(int argc, char **argv){
  fprintf(SE,"2D Grafika - Domaca uloha #2 v1.0\n(c) Miso Forisek\n");
  if (argc>1){
    if (strcmp(argv[1],"--help")==0) {
      fprintf(SE,"\nProgram nepodporuje ziadne parametre okrem --help.\n");
      fprintf(SE,"\nSyntax vstupu:\n");
      fprintf(SE,"xmin, ymin, xmax, ymax - suradnice 2 protilahlych vrcholov okna\n");
      fprintf(SE,"nasleduje niekolko useciek/priamok, zadavanych v tvare:\n");
      fprintf(SE,"U [resp. S] xs ys xf yf - Usecka (line Segment), spajajuca \n");
      fprintf(SE,"                          body [xs,ys] a [xf,yf]\n");
      fprintf(SE,"P [resp. L] a b c - Priamka (Line), dana rovnicou ax+by+c = 0\n");
      fprintf(SE,"E - end, t.j. koniec\n\n");
/*      fprintf(SE,"Program zostroji vyslednu transformaciu, zobrazi nou");
      fprintf(SE," polygon\n a vypise transformovane suradnice vrcholov\n");
      fprintf(SE,"v poradi, v akom boli zadane na vstupe.\n");*/
      fprintf(SE,"Okrem toho vytvori subor hw2.ps s grafickym vystupom.\n");
      return 0;
    } else {
      fprintf(SE,"Neznamy parameter. Pouzi --help\n");
      return 0;
    }
  }

  F=fopen("hw2.ps","wt");
  
  scanf("%lf %lf %lf %lf ",&xmin,&ymin,&xmax,&ymax);
  if (xmin>xmax) { pom=xmax; xmax=xmin; xmin=pom; }
  if (ymin>ymax) { pom=ymax; ymax=ymin; ymin=pom; }
  if ((xmin==xmax) || (ymin==ymax)) {
    printf("\nClipping window is degenerated to a line/point.\n");
    return 0;
  }
  printf("\nClipping into the window: [%.2f,%.2f]-[%.2f,%.2f]\n\n",xmin,ymin,xmax,ymax);

  WritePSHeader();

  scanf("%c ",&T); if (T>96) T-=32; 
  while (T!='E') {
    if ((T=='U')||(T=='S')) {
      scanf("%lf %lf %lf %lf ",&xa,&ya,&xb,&yb);
      res=CohenSutherland(xa,ya,xb,yb,&xca,&yca,&xcb,&ycb);
      switch (res) {
        case 0:
          printf("[%.2f,%.2f]-[%.2f,%.2f]\n  line segment completely outside.\n",xa,ya,xb,yb);

          fprintf(F,"gsave [5] 0 setdash\n");
          fprintf(F,"newpath %.5f %.5f moveto %.5f %.5f lineto stroke grestore\n",
                  PSScaledx(xa),PSScaledy(ya),PSScaledx(xb),PSScaledy(yb));
          break;
        case 1:
          printf("[%.2f,%.2f]-[%.2f,%.2f]\n  line segment clipped",xa,ya,xb,yb);
          printf(" to [%.2f,%.2f]-[%.2f,%.2f].\n",xca,yca,xcb,ycb);
          
          fprintf(F,"gsave [5] 0 setdash\n");
          fprintf(F,"newpath %.5f %.5f moveto %.5f %.5f lineto stroke grestore\n",
                  PSScaledx(xa),PSScaledy(ya),PSScaledx(xb),PSScaledy(yb));
          fprintf(F,"newpath %.5f %.5f moveto %.5f %.5f lineto stroke\n",
                  PSScaledx(xca),PSScaledy(yca),PSScaledx(xcb),PSScaledy(ycb));
          break;
        case 2:
        printf("[%.2f,%.2f]-[%.2f,%.2f]\n  line segment completely inside.\n",xa,ya,xb,yb);
        fprintf(F,"newpath %.5f %.5f moveto %.5f %.5f lineto stroke\n",
                PSScaledx(xca),PSScaledy(yca),PSScaledx(xcb),PSScaledy(ycb));
      }
    }
    if ((T=='P')||(T=='L')) {
      scanf("%lf %lf %lf ",&aa,&bb,&cc);
      if ((fabs(aa)<EPS)&&(fabs(bb)<EPS)) {
        printf("%.2f*x + %.2f*y + %.2f = 0\n  is not a correct line equation.\n",aa,bb,cc);
      } else {
        /* aproximujeme ju dost dlhou useckou */
        if (fabs(aa)<EPS) {
          /* vodorovna */
          ya=yb=-(cc/bb);
          xa=xmin-(xmax-xmin); xb=xmax+(xmax-xmin);
          res=CohenSutherland(xa,ya,xb,yb,&xca,&yca,&xcb,&ycb);
          switch (res) {
            case 0:
              printf("%.2f*x + %.2f*y + %.2f = 0\n  line completely outside.\n",aa,bb,cc);
          
              fprintf(F,"gsave [5] 0 setdash\n");
              fprintf(F,"newpath %.5f %.5f moveto %.5f %.5f lineto stroke grestore\n",
                      PSScaledx(xa),PSScaledy(ya),PSScaledx(xb),PSScaledy(yb));
              break;
            case 1:
              printf("%.2f*x + %.2f*y + %.2f = 0\n  line clipped",aa,bb,cc);
              printf(" to [%.2f,%.2f]-[%.2f,%.2f].\n",xca,yca,xcb,ycb);
          
              fprintf(F,"gsave [5] 0 setdash\n");
              fprintf(F,"newpath %.5f %.5f moveto %.5f %.5f lineto stroke grestore\n",
                      PSScaledx(xa),PSScaledy(ya),PSScaledx(xb),PSScaledy(yb));
              fprintf(F,"newpath %.5f %.5f moveto %.5f %.5f lineto stroke\n",
                      PSScaledx(xca),PSScaledy(yca),PSScaledx(xcb),PSScaledy(ycb));
              break;
          }
        } else {
          /* nie vodorovna */
          ya=ymin-(ymax-ymin); yb=ymax+(ymax-ymin); 
          xa=-(bb*ya/aa)-(cc/aa); xb=-(bb*yb/aa)-(cc/aa);
          res=CohenSutherland(xa,ya,xb,yb,&xca,&yca,&xcb,&ycb);
          switch (res) {
            case 0:
              printf("%.2f*x + %.2f*y + %.2f = 0\n  line completely outside.\n",aa,bb,cc);
          
              fprintf(F,"gsave [5] 0 setdash\n");
              fprintf(F,"newpath %.5f %.5f moveto %.5f %.5f lineto stroke grestore\n",
                      PSScaledx(xa),PSScaledy(ya),PSScaledx(xb),PSScaledy(yb));
              break;
            case 1:
              printf("%.2f*x + %.2f*y + %.2f = 0\n  line clipped",aa,bb,cc);
              printf(" to [%.2f,%.2f]-[%.2f,%.2f].\n",xca,yca,xcb,ycb);
          
              fprintf(F,"gsave [5] 0 setdash\n");
              fprintf(F,"newpath %.5f %.5f moveto %.5f %.5f lineto stroke grestore\n",
                      PSScaledx(xa),PSScaledy(ya),PSScaledx(xb),PSScaledy(yb));
              fprintf(F,"newpath %.5f %.5f moveto %.5f %.5f lineto stroke\n",
                      PSScaledx(xca),PSScaledy(yca),PSScaledx(xcb),PSScaledy(ycb));
              break;
          }
        }
      }
    }
    scanf("%c ",&T); if (T>96) T-=32; 
  }

  fprintf(F,"showpage\n");
  fclose(F);

  return 0;
}


/*

















typedef struct { double x,y; } bod_2d;
typedef struct { bod_2d *op,*tp; int N; } poly;

poly P, DispP;
int i,j;

void WritePSHeader(void){

  fprintf(F,"%%!PS (c) MisoF\n\n");
  
  fprintf(F,"%% ---------\n%% definicie\n%% ---------\n\n");
  fprintf(F,"/cm {72 2.54 div mul} def\n/pis {newpath moveto show} def\n\n");

  fprintf(F,"%% --------\n%% hlavicka\n%% --------\n\n");
  fprintf(F,"/Times-Roman findfont 24 scalefont setfont\n");
  fprintf(F,"(2D grafika - homework #1) 174 800 pis\n\n");

  fprintf(F,"/Times-Roman findfont 16 scalefont setfont\n1.4 setlinewidth\n\n");
  fprintf(F,"(\\(c\\) MisoF) 520 10 pis\n\n");


  fprintf(F,"0.8 0.0 0.0 setrgbcolor\n");
  fprintf(F,"newpath 50 750 moveto 50 770 lineto 70 770 lineto");
  fprintf(F," closepath fill\n");
  fprintf(F,"0.0 0.0 0.0 setrgbcolor\n");
  fprintf(F,"newpath 50 750 moveto 50 770 lineto 70 770 lineto");
  fprintf(F," closepath stroke\n");
  fprintf(F,"0 setgray (Original shape) 80 755 pis\n\n");

  fprintf(F,"0.0 0.8 0.0 setrgbcolor\n");
  fprintf(F,"newpath 70 725 moveto 50 745 lineto 70 745 lineto");
  fprintf(F," closepath fill\n");
  fprintf(F,"0.0 0.0 0.0 setrgbcolor\n");
  fprintf(F,"newpath 70 725 moveto 50 745 lineto 70 745 lineto");
  fprintf(F," closepath stroke\n");
  fprintf(F,"0 setgray (Transformed shape) 80 730 pis\n\n");

  fprintf(F,"%% -------------------\n%% popis transformacie\n");
  fprintf(F,"%% -------------------\n\n");
  fprintf(F,"(Transformation:) 300 755 pis\n");
}

void ScaleCoordinatesForDisplay(void){
  double minx,maxx,miny,maxy;
  int i;
  
  minx=maxx=P.op[0].x;
  miny=maxy=P.op[0].y;
  for (i=1;i<P.N;i++){
    if (P.op[i].x<minx) minx=P.op[i].x; if (P.op[i].x>maxx) maxx=P.op[i].x;
    if (P.op[i].y<miny) miny=P.op[i].y; if (P.op[i].y>maxy) maxy=P.op[i].y;
  }
  for (i=0;i<P.N;i++){
    if (P.tp[i].x<minx) minx=P.tp[i].x; if (P.tp[i].x>maxx) maxx=P.tp[i].x;
    if (P.tp[i].y<miny) miny=P.tp[i].y; if (P.tp[i].y>maxy) maxy=P.tp[i].y;
  }

  if (minx==maxx) { for (i=0;i<P.N;i++) DispP.op[i].x=250; } 
   else{for(i=0;i<P.N;i++)DispP.op[i].x=(P.op[i].x-minx)*(450/(maxx-minx))+50;}
  if (miny==maxy) { for (i=0;i<P.N;i++) DispP.op[i].y=250; } 
   else{for(i=0;i<P.N;i++)DispP.op[i].y=(P.op[i].y-miny)*(450/(maxy-miny))+50;}
   
  if (minx==maxx) { for (i=0;i<P.N;i++) DispP.tp[i].x=250; } 
   else{for(i=0;i<P.N;i++)DispP.tp[i].x=(P.tp[i].x-minx)*(450/(maxx-minx))+50;}
  if (miny==maxy) { for (i=0;i<P.N;i++) DispP.tp[i].y=250; } 
   else{for(i=0;i<P.N;i++)DispP.tp[i].y=(P.tp[i].y-miny)*(450/(maxy-miny))+50;}
}

void WritePS(void){
  int i;
  
  fprintf(F,"\n%% ---------------\n%% povodny polygon\n%% ---------------\n\n");
  fprintf(F,"/poly1 {\n  newpath\n");
  fprintf(F,"  %.5f %.5f moveto\n",DispP.op[0].x,DispP.op[0].y);
  for (i=1;i<P.N;i++) fprintf(F,"  %.5f %.5f lineto\n",DispP.op[i].x,DispP.op[i].y);
  fprintf(F,"  closepath\n} def\n\n");

  fprintf(F,"0.8 0.0 0.0 setrgbcolor\npoly1 fill\n");
  fprintf(F,"0.0 0.0 0.0 setrgbcolor\npoly1 stroke\n\n");

  fprintf(F,"0 setgray\n");
  for (i=0;i<P.N;i++) 
    fprintf(F,"([%.1f,%.1f]) %.5f %.5f pis\n",P.op[i].x,P.op[i].y,DispP.op[i].x+5,DispP.op[i].y+5);

  fprintf(F,"\n%% ------------\n%% novy polygon\n%% ------------\n\n");
  fprintf(F,"/poly2 {\n  newpath\n");
  fprintf(F,"  %.5f %.5f moveto\n",DispP.tp[0].x,DispP.tp[0].y);
  for (i=1;i<P.N;i++) fprintf(F,"  %.5f %.5f lineto\n",DispP.tp[i].x,DispP.tp[i].y);
  fprintf(F,"  closepath\n} def\n\n");

  fprintf(F,"0.0 0.8 0.0 setrgbcolor\npoly2 fill\n");
  fprintf(F,"0.0 0.0 0.0 setrgbcolor\npoly2 stroke\n\n");

  fprintf(F,"0 setgray\n");
  for (i=0;i<P.N;i++) 
    fprintf(F,"([%.1f,%.1f]) %.5f %.5f pis\n",P.tp[i].x,P.tp[i].y,DispP.tp[i].x+5,DispP.tp[i].y+5);

  fprintf(F,"\nshowpage\n");
}


  F=fopen("hw1.ps","wt");
  WritePSHeader();

  Transform(&P);

  for (i=0;i<P.N;i++) printf("%.6f %.6f\n",P.tp[i].x,P.tp[i].y);
  ScaleCoordinatesForDisplay();
  WritePS();
  
  free(P.op);
  free(P.tp);
  free(DispP.op);
  free(DispP.tp);
  fclose(F);
  return 0;
}
*/
