#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#define SE stderr
#define PI 3.141592654
#define EPS 0.000001
#define MAXROW 10000

FILE *F,*FM;
double Matrix[9][9];
int M,N; // matrix size
int X,Y; // image size
int C;   // shades of gray
double IMG[9][MAXROW];

void ReadMatrix() {
  int i,j;
  
  fscanf(FM,"%d %d ",&M,&N);
  for (i=0;i<M;i++) for (j=0;j<N;j++) fscanf(FM,"%lf ",&Matrix[i][j]);
  M=(M-1)/2; N=(N-1)/2; // matica bude od -M do M, od -N do N
}

void ReadHeader(){
  int mamx,mamy,mamc,kelo;
  char s[100];
  char *ss;
  
  fgets(s,100,F);
  ss=strstr(s,"P2");
  if (s!=ss) {
    fprintf(SE,"Image is not an Ascii-PGM file! (P5 instead of P2?)\n");
    exit(1);
  }
  mamx=mamy=mamc=0;
  while (!mamc) {
    fgets(s,100,F); ss=strstr(s,"#"); if (ss!=NULL) ss[0]=0;
    if (!mamx) {
      kelo=sscanf(s,"%d %d %d",&X,&Y,&C);
      if (kelo>=1) mamx=1; if (kelo>=2) mamy=1; if (kelo==3) mamc=1;
    } else { if (!mamy) {
      kelo=sscanf(s,"%d %d",&Y,&C);
      if (kelo>=1) mamy=1; if (kelo>=2) mamc=1;
    } else {
      kelo=sscanf(s,"%d",&C); if (kelo>0) mamc=1;
    }}
  }
  printf("P2\n# Created by MisoF\n%d %d\n%d\n",X,Y,C);
}

void ReadRow(int no){
  int i; 
  if (no>=Y) return;
  for (i=0;i<X;i++) fscanf(F,"%lf ",&IMG[no%9][i]);
}

int IsOk(int yy, int xx){
  if (xx<0) return 0; if (xx>=X) return 0;
  if (yy<0) return 0; if (yy>=Y) return 0;
  return 1;
}

void ComputeRow(int no){
  int xx,i,j;
  double factor;
  double newval;
  int inewval;
  
  for (xx=0;xx<X;xx++){
    factor=0;
    for (i=-M;i<=M;i++) for (j=-N;j<=N;j++) factor+=IsOk(no+i,xx+j);
    factor/=((2*M+1)*(2*N+1));
    factor=1/factor;
    newval=0;
    for (i=-M;i<=M;i++) for (j=-N;j<=N;j++) 
      if (IsOk(no+i,xx+j)) newval+=factor*Matrix[i+M][j+N]*IMG[(no+i)%9][xx+j];
    newval+=0.5; inewval=(int)newval;
    if (inewval<0) inewval=0; if (inewval>C) inewval=C;
    printf("%d\n",inewval);
  }
}

void ApplyMatrix() {
  int i,row;
  
  ReadHeader();
//  if (Y<(2*M+1)) { SolveSpecialCase(); return; } // menej riadkov ako matica
  for (i=0;i<M;i++) ReadRow(i);
  for (row=0;row<Y;row++) {
    // docitame dalsi riadok
    ReadRow(row+M);
    // mame riadky +-M v poli (ak existuju)
    ComputeRow(row);
  }
}

int main(int argc, char **argv){
  fprintf(SE,"2D Grafika - Domaca uloha #4 v1.0\n(c) Miso Forisek\n");
  if (((argc>1)&&(strcmp(argv[1],"--help")==0))||(argc==1)){
    fprintf(SE,"Pouzitie:\n./%s",argv[0]);
    fprintf(SE," [subor_s_maticou] [obrazok.pgm] >[vystup.pgm]\n\n");
     
    fprintf(SE,"Subor s maticou ma mat nasledovnu strukturu:\n");
    fprintf(SE,"  M N -- pocet riadkov a stlpcov matice (1,3,5,7,9)\n");
    fprintf(SE,"  MxN realnych cisel -- policka matice po riadkoch\n\n");

    fprintf(SE,"Obrazok vo formate Ascii-PGM moze mat takmer lub. rozmery\n\n");
    return 0;
  } 
  if (argc<3) {
    fprintf(SE,"Malo parametrov!\n\n");
    return 1;
  }
  
  FM=fopen(argv[1],"rt"); F=fopen(argv[2],"rt");

  ReadMatrix();
  ApplyMatrix(); 

  fclose(F); fclose(FM);

  return 0;
}

